/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_CANSERVICE_CANSERVICE_SKELETON_H
#define JAC_AP_CANSERVICE_CANSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/canservice/canservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace canservice {
namespace skeleton {
namespace events
{
    using INS_CAN2SWC_100HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::INS_CAN2SWC_100HZ>;
    using IMU_CAN2SWC_100HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::IMU_CAN2SWC_100HZ>;
    using GNSS_CAN2SWC_5HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::GNSS_CAN2SWC_5HZ>;
    using VEHICLE_CAN2SWC_50HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::VEHICLE_CAN2SWC_50HZ>;
    using VEHICLE_CAN2SWC_10HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::VEHICLE_CAN2SWC_10HZ>;
    using CAN2SWCEvent = ara::com::internal::skeleton::event::EventAdapter<::CAN2SWC>;
    using VEHICLE_CAN_MCU2SOC_50HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_50HZ>;
    using VEHICLE_CAN_MCU2SOC_10HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_10HZ>;
    using VEHICLE_CAN_MCU2SOC_100HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_100HZ>;
    using VEHICLE_CAN_MCU2SOC_2HZ_Event = ara::com::internal::skeleton::event::EventAdapter<::newcan::VEHICLE_CAN_MCU2SOC_2HZ>;
    static constexpr ara::com::internal::EntityId CANServiceINS_CAN2SWC_100HZ_EventId = 6447U; //INS_CAN2SWC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceIMU_CAN2SWC_100HZ_EventId = 21038U; //IMU_CAN2SWC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceGNSS_CAN2SWC_5HZ_EventId = 21370U; //GNSS_CAN2SWC_5HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN2SWC_50HZ_EventId = 25430U; //VEHICLE_CAN2SWC_50HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN2SWC_10HZ_EventId = 17496U; //VEHICLE_CAN2SWC_10HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceCAN2SWCEventId = 15226U; //CAN2SWCEvent_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId = 16667U; //VEHICLE_CAN_MCU2SOC_50HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId = 10596U; //VEHICLE_CAN_MCU2SOC_10HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId = 64925U; //VEHICLE_CAN_MCU2SOC_100HZ_Event_event_hash
    static constexpr ara::com::internal::EntityId CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId = 26133U; //VEHICLE_CAN_MCU2SOC_2HZ_Event_event_hash
}

namespace methods
{
}

namespace fields
{
}

class CANServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultINS_CAN2SWC_100HZ_Event = INS_CAN2SWC_100HZ_Event.Initialize();
        ThrowError(resultINS_CAN2SWC_100HZ_Event);
        const ara::core::Result<void> resultIMU_CAN2SWC_100HZ_Event = IMU_CAN2SWC_100HZ_Event.Initialize();
        ThrowError(resultIMU_CAN2SWC_100HZ_Event);
        const ara::core::Result<void> resultGNSS_CAN2SWC_5HZ_Event = GNSS_CAN2SWC_5HZ_Event.Initialize();
        ThrowError(resultGNSS_CAN2SWC_5HZ_Event);
        const ara::core::Result<void> resultVEHICLE_CAN2SWC_50HZ_Event = VEHICLE_CAN2SWC_50HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN2SWC_50HZ_Event);
        const ara::core::Result<void> resultVEHICLE_CAN2SWC_10HZ_Event = VEHICLE_CAN2SWC_10HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN2SWC_10HZ_Event);
        const ara::core::Result<void> resultCAN2SWCEvent = CAN2SWCEvent.Initialize();
        ThrowError(resultCAN2SWCEvent);
        const ara::core::Result<void> resultVEHICLE_CAN_MCU2SOC_50HZ_Event = VEHICLE_CAN_MCU2SOC_50HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN_MCU2SOC_50HZ_Event);
        const ara::core::Result<void> resultVEHICLE_CAN_MCU2SOC_10HZ_Event = VEHICLE_CAN_MCU2SOC_10HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN_MCU2SOC_10HZ_Event);
        const ara::core::Result<void> resultVEHICLE_CAN_MCU2SOC_100HZ_Event = VEHICLE_CAN_MCU2SOC_100HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN_MCU2SOC_100HZ_Event);
        const ara::core::Result<void> resultVEHICLE_CAN_MCU2SOC_2HZ_Event = VEHICLE_CAN_MCU2SOC_2HZ_Event.Initialize();
        ThrowError(resultVEHICLE_CAN_MCU2SOC_2HZ_Event);
    }

    CANServiceSkeleton& operator=(const CANServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceId, mode)),
              INS_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              IMU_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              GNSS_CAN2SWC_5HZ_Event(ptr->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN2SWC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN2SWC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              CAN2SWCEvent(ptr->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN_MCU2SOC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN_MCU2SOC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN_MCU2SOC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
              VEHICLE_CAN_MCU2SOC_2HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec, mode)),
              INS_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              IMU_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              GNSS_CAN2SWC_5HZ_Event(ptr->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN2SWC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN2SWC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              CAN2SWCEvent(ptr->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN_MCU2SOC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN_MCU2SOC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN_MCU2SOC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
              VEHICLE_CAN_MCU2SOC_2HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer, mode)),
              INS_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              IMU_CAN2SWC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              GNSS_CAN2SWC_5HZ_Event(ptr->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN2SWC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN2SWC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              CAN2SWCEvent(ptr->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN_MCU2SOC_50HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN_MCU2SOC_10HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN_MCU2SOC_100HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
              VEHICLE_CAN_MCU2SOC_2HZ_Event(ptr->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              INS_CAN2SWC_100HZ_Event(std::move(other.INS_CAN2SWC_100HZ_Event)),
              IMU_CAN2SWC_100HZ_Event(std::move(other.IMU_CAN2SWC_100HZ_Event)),
              GNSS_CAN2SWC_5HZ_Event(std::move(other.GNSS_CAN2SWC_5HZ_Event)),
              VEHICLE_CAN2SWC_50HZ_Event(std::move(other.VEHICLE_CAN2SWC_50HZ_Event)),
              VEHICLE_CAN2SWC_10HZ_Event(std::move(other.VEHICLE_CAN2SWC_10HZ_Event)),
              CAN2SWCEvent(std::move(other.CAN2SWCEvent)),
              VEHICLE_CAN_MCU2SOC_50HZ_Event(std::move(other.VEHICLE_CAN_MCU2SOC_50HZ_Event)),
              VEHICLE_CAN_MCU2SOC_10HZ_Event(std::move(other.VEHICLE_CAN_MCU2SOC_10HZ_Event)),
              VEHICLE_CAN_MCU2SOC_100HZ_Event(std::move(other.VEHICLE_CAN_MCU2SOC_100HZ_Event)),
              VEHICLE_CAN_MCU2SOC_2HZ_Event(std::move(other.VEHICLE_CAN_MCU2SOC_2HZ_Event)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                INS_CAN2SWC_100HZ_Event = std::move(other.INS_CAN2SWC_100HZ_Event);
                IMU_CAN2SWC_100HZ_Event = std::move(other.IMU_CAN2SWC_100HZ_Event);
                GNSS_CAN2SWC_5HZ_Event = std::move(other.GNSS_CAN2SWC_5HZ_Event);
                VEHICLE_CAN2SWC_50HZ_Event = std::move(other.VEHICLE_CAN2SWC_50HZ_Event);
                VEHICLE_CAN2SWC_10HZ_Event = std::move(other.VEHICLE_CAN2SWC_10HZ_Event);
                CAN2SWCEvent = std::move(other.CAN2SWCEvent);
                VEHICLE_CAN_MCU2SOC_50HZ_Event = std::move(other.VEHICLE_CAN_MCU2SOC_50HZ_Event);
                VEHICLE_CAN_MCU2SOC_10HZ_Event = std::move(other.VEHICLE_CAN_MCU2SOC_10HZ_Event);
                VEHICLE_CAN_MCU2SOC_100HZ_Event = std::move(other.VEHICLE_CAN_MCU2SOC_100HZ_Event);
                VEHICLE_CAN_MCU2SOC_2HZ_Event = std::move(other.VEHICLE_CAN_MCU2SOC_2HZ_Event);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::INS_CAN2SWC_100HZ_Event GetINS_CAN2SWC_100HZ_Event()
        {
            return std::move(INS_CAN2SWC_100HZ_Event);
        }
        events::IMU_CAN2SWC_100HZ_Event GetIMU_CAN2SWC_100HZ_Event()
        {
            return std::move(IMU_CAN2SWC_100HZ_Event);
        }
        events::GNSS_CAN2SWC_5HZ_Event GetGNSS_CAN2SWC_5HZ_Event()
        {
            return std::move(GNSS_CAN2SWC_5HZ_Event);
        }
        events::VEHICLE_CAN2SWC_50HZ_Event GetVEHICLE_CAN2SWC_50HZ_Event()
        {
            return std::move(VEHICLE_CAN2SWC_50HZ_Event);
        }
        events::VEHICLE_CAN2SWC_10HZ_Event GetVEHICLE_CAN2SWC_10HZ_Event()
        {
            return std::move(VEHICLE_CAN2SWC_10HZ_Event);
        }
        events::CAN2SWCEvent GetCAN2SWCEvent()
        {
            return std::move(CAN2SWCEvent);
        }
        events::VEHICLE_CAN_MCU2SOC_50HZ_Event GetVEHICLE_CAN_MCU2SOC_50HZ_Event()
        {
            return std::move(VEHICLE_CAN_MCU2SOC_50HZ_Event);
        }
        events::VEHICLE_CAN_MCU2SOC_10HZ_Event GetVEHICLE_CAN_MCU2SOC_10HZ_Event()
        {
            return std::move(VEHICLE_CAN_MCU2SOC_10HZ_Event);
        }
        events::VEHICLE_CAN_MCU2SOC_100HZ_Event GetVEHICLE_CAN_MCU2SOC_100HZ_Event()
        {
            return std::move(VEHICLE_CAN_MCU2SOC_100HZ_Event);
        }
        events::VEHICLE_CAN_MCU2SOC_2HZ_Event GetVEHICLE_CAN_MCU2SOC_2HZ_Event()
        {
            return std::move(VEHICLE_CAN_MCU2SOC_2HZ_Event);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = INS_CAN2SWC_100HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = IMU_CAN2SWC_100HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = GNSS_CAN2SWC_5HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN2SWC_50HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN2SWC_10HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = CAN2SWCEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN_MCU2SOC_50HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN_MCU2SOC_10HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN_MCU2SOC_100HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VEHICLE_CAN_MCU2SOC_2HZ_Event.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::INS_CAN2SWC_100HZ_Event INS_CAN2SWC_100HZ_Event;
        events::IMU_CAN2SWC_100HZ_Event IMU_CAN2SWC_100HZ_Event;
        events::GNSS_CAN2SWC_5HZ_Event GNSS_CAN2SWC_5HZ_Event;
        events::VEHICLE_CAN2SWC_50HZ_Event VEHICLE_CAN2SWC_50HZ_Event;
        events::VEHICLE_CAN2SWC_10HZ_Event VEHICLE_CAN2SWC_10HZ_Event;
        events::CAN2SWCEvent CAN2SWCEvent;
        events::VEHICLE_CAN_MCU2SOC_50HZ_Event VEHICLE_CAN_MCU2SOC_50HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_10HZ_Event VEHICLE_CAN_MCU2SOC_10HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_100HZ_Event VEHICLE_CAN_MCU2SOC_100HZ_Event;
        events::VEHICLE_CAN_MCU2SOC_2HZ_Event VEHICLE_CAN_MCU2SOC_2HZ_Event;
    };
    explicit CANServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceId, mode)),
          INS_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          IMU_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          GNSS_CAN2SWC_5HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN2SWC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN2SWC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          CAN2SWCEvent(skeletonAdapter->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit CANServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec, mode)),
          INS_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          IMU_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          GNSS_CAN2SWC_5HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN2SWC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN2SWC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          CAN2SWCEvent(skeletonAdapter->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit CANServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer, mode)),
          INS_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceINS_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          IMU_CAN2SWC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceIMU_CAN2SWC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          GNSS_CAN2SWC_5HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceGNSS_CAN2SWC_5HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN2SWC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN2SWC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN2SWC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          CAN2SWCEvent(skeletonAdapter->GetSkeleton(), events::CANServiceCAN2SWCEventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_50HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_10HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_100HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(skeletonAdapter->GetSkeleton(), events::CANServiceVEHICLE_CAN_MCU2SOC_2HZ_EventId, ::jac::ap::canservice::CANService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    CANServiceSkeleton(const CANServiceSkeleton&) = delete;

    CANServiceSkeleton(CANServiceSkeleton&&) = default;
    CANServiceSkeleton& operator=(CANServiceSkeleton&&) = default;
    CANServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          INS_CAN2SWC_100HZ_Event(token.GetINS_CAN2SWC_100HZ_Event()),
          IMU_CAN2SWC_100HZ_Event(token.GetIMU_CAN2SWC_100HZ_Event()),
          GNSS_CAN2SWC_5HZ_Event(token.GetGNSS_CAN2SWC_5HZ_Event()),
          VEHICLE_CAN2SWC_50HZ_Event(token.GetVEHICLE_CAN2SWC_50HZ_Event()),
          VEHICLE_CAN2SWC_10HZ_Event(token.GetVEHICLE_CAN2SWC_10HZ_Event()),
          CAN2SWCEvent(token.GetCAN2SWCEvent()),
          VEHICLE_CAN_MCU2SOC_50HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_50HZ_Event()),
          VEHICLE_CAN_MCU2SOC_10HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_10HZ_Event()),
          VEHICLE_CAN_MCU2SOC_100HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_100HZ_Event()),
          VEHICLE_CAN_MCU2SOC_2HZ_Event(token.GetVEHICLE_CAN_MCU2SOC_2HZ_Event()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~CANServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&CANServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::INS_CAN2SWC_100HZ_Event INS_CAN2SWC_100HZ_Event;
    events::IMU_CAN2SWC_100HZ_Event IMU_CAN2SWC_100HZ_Event;
    events::GNSS_CAN2SWC_5HZ_Event GNSS_CAN2SWC_5HZ_Event;
    events::VEHICLE_CAN2SWC_50HZ_Event VEHICLE_CAN2SWC_50HZ_Event;
    events::VEHICLE_CAN2SWC_10HZ_Event VEHICLE_CAN2SWC_10HZ_Event;
    events::CAN2SWCEvent CAN2SWCEvent;
    events::VEHICLE_CAN_MCU2SOC_50HZ_Event VEHICLE_CAN_MCU2SOC_50HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_10HZ_Event VEHICLE_CAN_MCU2SOC_10HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_100HZ_Event VEHICLE_CAN_MCU2SOC_100HZ_Event;
    events::VEHICLE_CAN_MCU2SOC_2HZ_Event VEHICLE_CAN_MCU2SOC_2HZ_Event;
};
} // namespace skeleton
} // namespace canservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_CANSERVICE_CANSERVICE_SKELETON_H
